@@ -128,6 +128,10 @@ type ClusterDeploymentSpec struct {
 	// time since the cluster last came out of hibernation.
 	// +optional
 	HibernateAfter *metav1.Duration `json:"hibernateAfter,omitempty"`
+
+	// InstallAttemptsLimit is the maximum number of times Hive will attempt to install the cluster.
+	// +optional
+	InstallAttemptsLimit *int32 `json:"installAttemptsLimit,omitempty"`
 }
 
 // Provisioning contains settings used only for initial cluster provisioning.
@@ -287,14 +291,14 @@ const (
 	// API URL override.
 	ActiveAPIURLOverrideCondition ClusterDeploymentConditionType = "ActiveAPIURLOverride"
 
-	// InstallFailingCondition indicates that a failure has been detected and we will attempt to offer some
-	// information as to why in the reason.
-	InstallFailingCondition ClusterDeploymentConditionType = "InstallFailing"
-
 	// DNSNotReadyCondition indicates that the the DNSZone object created for the clusterDeployment
 	// (ie manageDNS==true) has not yet indicated that the DNS zone is successfully responding to queries.
 	DNSNotReadyCondition ClusterDeploymentConditionType = "DNSNotReady"
 
+	// InstallImagesResolvedCondition indicates that the the install images for the clusterDeployment
+	// have been not been resolved. This usually includes the installer and OpenShift cli images.
+	InstallImagesNotResolvedCondition ClusterDeploymentConditionType = "InstallImagesNotResolved"
+
 	// ProvisionFailedCondition indicates that a provision failed
 	ProvisionFailedCondition ClusterDeploymentConditionType = "ProvisionFailed"
 
@@ -313,6 +317,12 @@ const (
 
 	// DeprovisionLaunchErrorCondition is set when a cluster deprovision fails to launch.
 	DeprovisionLaunchErrorCondition ClusterDeploymentConditionType = "DeprovisionLaunchError"
+
+	// ProvisionStoppedCondition is set when cluster provisioning is stopped
+	ProvisionStoppedCondition ClusterDeploymentConditionType = "ProvisionStopped"
+
+	// AuthenticationFailureCondition is true when platform credentials cannot be used because of authentication failure
+	AuthenticationFailureClusterDeploymentCondition ClusterDeploymentConditionType = "AuthenticationFailure"
 )
 
 // AllClusterDeploymentConditions is a slice containing all condition types. This can be used for dealing with
@@ -324,7 +334,6 @@ var AllClusterDeploymentConditions = []ClusterDeploymentConditionType{
 	IngressCertificateNotFoundCondition,
 	UnreachableCondition,
 	ActiveAPIURLOverrideCondition,
-	InstallFailingCondition,
 	DNSNotReadyCondition,
 	ProvisionFailedCondition,
 	SyncSetFailedCondition,
@@ -358,6 +367,9 @@ const (
 	// FailedToStartHibernationReason is used when there was an error starting machines
 	// to leave hibernation
 	FailedToStartHibernationReason = "FailedToStart"
+	// SyncSetsNotAppliedReason is used as the reason when SyncSets have not yet been applied
+	// for the cluster based on ClusterSync.Status.FirstSucessTime
+	SyncSetsNotAppliedReason = "SyncSetsNotApplied"
 )
 
 // +genclient
