@@ -127,20 +127,10 @@ type ReconcileDNSZone struct {
 // Reconcile reads that state of the cluster for a DNSZone object and makes changes based on the state read
 // and what is in the DNSZone.Spec
 func (r *ReconcileDNSZone) Reconcile(request reconcile.Request) (reconcile.Result, error) {
-	start := time.Now()
-	dnsLog := r.logger.WithFields(log.Fields{
-		"controller": ControllerName,
-		"dnszone":    request.Name,
-		"namespace":  request.Namespace,
-	})
-
-	// For logging, we need to see when the reconciliation loop starts and ends.
+	dnsLog := controllerutils.BuildControllerLogger(ControllerName, "dnsZone", request.NamespacedName)
 	dnsLog.Info("reconciling dns zone")
-	defer func() {
-		dur := time.Since(start)
-		hivemetrics.MetricControllerReconcileTime.WithLabelValues(ControllerName.String()).Observe(dur.Seconds())
-		dnsLog.WithField("elapsed", dur).Info("reconcile complete")
-	}()
+	recobsrv := hivemetrics.NewReconcileObserver(ControllerName, dnsLog)
+	defer recobsrv.ObserveControllerReconcileTime()
 
 	// Fetch the DNSZone object
 	desiredState := &hivev1.DNSZone{}
